pub struct AppsyncResponse { /* private fields */ }Expand description
Response structure returned to AWS AppSync from a Lambda resolver.
Can contain either successful data or error information, but not both. Should be constructed using From implementations for either Value (success) or AppsyncError (failure).
Examples
// Success response
let response: AppsyncResponse = json!({ "id": 123 }).into();
// Error response
let error = AppsyncError::new("NotFound", "Resource not found");
let response: AppsyncResponse = error.into();Implementations
Sourceimpl AppsyncResponse
impl AppsyncResponse
Returns an unauthorized error response
This creates a standard unauthorized error response for when a request lacks proper authentication.
Examples
let response = AppsyncResponse::unauthorized();Sourcepub fn data(&self) -> Option<&Value>
pub fn data(&self) -> Option<&Value>
Returns a reference to the response data, if present
Examples
let response = AppsyncResponse::from(json!({"user": "Alice"}));
assert!(response.data().is_some());Sourcepub fn error(&self) -> Option<&AppsyncError>
pub fn error(&self) -> Option<&AppsyncError>
Returns a reference to the response error, if present
Examples
let error = AppsyncError::new("NotFound", "User not found");
let response = AppsyncResponse::from(error);
assert!(response.error().is_some());Trait Implementations
Sourceimpl Debug for AppsyncResponse
impl Debug for AppsyncResponse
Sourceimpl From<AppsyncError> for AppsyncResponse
impl From<AppsyncError> for AppsyncResponse
Sourcefn from(value: AppsyncError) -> Self
fn from(value: AppsyncError) -> Self
Converts to this type from the input type.
Sourceimpl From<Value> for AppsyncResponse
impl From<Value> for AppsyncResponse